@@ -22,6 +22,7 @@
 #include "mainwindow.h"
 #include "desktopwindow.h"
 #include <QDBusConnection>
+#include <QDBusConnectionInterface>
 #include <QDBusInterface>
 #include <QDir>
 #include <QVector>
@@ -40,12 +41,15 @@
 #include <sys/socket.h>
 
 #include <libfm-qt/mountoperation.h>
+#include <libfm-qt/filepropsdialog.h>
 #include <libfm-qt/filesearchdialog.h>
 #include <libfm-qt/core/terminal.h>
 #include <libfm-qt/core/bookmarks.h>
 #include <libfm-qt/core/folderconfig.h>
+#include <libfm-qt/core/fileinfojob.h>
 
 #include "applicationadaptor.h"
+#include "applicationadaptorfreedesktopfilemanager.h"
 #include "preferencesdialog.h"
 #include "desktoppreferencesdialog.h"
 #include "autorundialog.h"
@@ -87,6 +91,9 @@ Application::Application(int& argc, char** argv):
     userDirsWatcher_(nullptr),
     lxqtRunning_(false) {
 
+    // mainly used to disable tab DND outside X11 because Wayland has problem with QDrag
+    isX11_ = QGuiApplication::platformName() == QStringLiteral("xcb");
+
     argc_ = argc;
     argv_ = argv;
 
@@ -121,6 +128,22 @@ Application::Application(int& argc, char** argv):
             delete lxqtSessionIface;
             lxqtSessionIface = nullptr;
         }
+
+
+        // We also try to register the service "org.freedesktop.FileManager1".
+        // We allow queuing of our request in case another file manager has already registered it.
+        static const QString fileManagerService = QStringLiteral("org.freedesktop.FileManager1");
+        connect(dbus.interface(), &QDBusConnectionInterface::serviceRegistered, this, [this](const QString& service) {
+                if(fileManagerService == service) {
+                    QDBusConnection dbus = QDBusConnection::sessionBus();
+                    disconnect(dbus.interface(), &QDBusConnectionInterface::serviceRegistered, this, nullptr);
+                    new ApplicationAdaptorFreeDesktopFileManager(this);
+                    if(!dbus.registerObject(QStringLiteral("/org/freedesktop/FileManager1"), this)) {
+                        qDebug() << "Can't register /org/freedesktop/FileManager1:" << dbus.lastError().message();
+                    }
+                }
+        });
+        dbus.interface()->registerService(fileManagerService, QDBusConnectionInterface::QueueService);
     }
     else {
         // an service of the same name is already registered.
@@ -165,10 +188,10 @@ bool Application::parseCommandLineArgs() {
     QCommandLineOption profileOption(QStringList() << QStringLiteral("p") << QStringLiteral("profile"), tr("Name of configuration profile"), tr("PROFILE"));
     parser.addOption(profileOption);
 
-    QCommandLineOption daemonOption(QStringList() << QStringLiteral("d") << QStringLiteral("daemon-mode"), tr("Run PCManFM as a daemon"));
+    QCommandLineOption daemonOption(QStringList() << QStringLiteral("d") << QStringLiteral("daemon-mode"), tr("Run PCManFM-Qt as a daemon"));
     parser.addOption(daemonOption);
 
-    QCommandLineOption quitOption(QStringList() << QStringLiteral("q") << QStringLiteral("quit"), tr("Quit PCManFM"));
+    QCommandLineOption quitOption(QStringList() << QStringLiteral("q") << QStringLiteral("quit"), tr("Quit PCManFM-Qt"));
     parser.addOption(quitOption);
 
     QCommandLineOption desktopOption(QStringLiteral("desktop"), tr("Launch desktop manager"));
@@ -177,7 +200,7 @@ bool Application::parseCommandLineArgs() {
     QCommandLineOption desktopOffOption(QStringLiteral("desktop-off"), tr("Turn off desktop manager if it's running"));
     parser.addOption(desktopOffOption);
 
-    QCommandLineOption desktopPrefOption(QStringLiteral("desktop-pref"), tr("Open desktop preference dialog on the page with the specified name"), tr("NAME"));
+    QCommandLineOption desktopPrefOption(QStringLiteral("desktop-pref"), tr("Open desktop preference dialog on the page with the specified name") + QStringLiteral("\n") + tr("NAME") + QStringLiteral("=(general|bg|slide|advanced)"), tr("NAME"));
     parser.addOption(desktopPrefOption);
 
     QCommandLineOption newWindowOption(QStringList() << QStringLiteral("n") << QStringLiteral("new-window"), tr("Open new window"));
@@ -192,7 +215,7 @@ bool Application::parseCommandLineArgs() {
     QCommandLineOption wallpaperModeOption(QStringLiteral("wallpaper-mode"), tr("Set mode of desktop wallpaper. MODE=(%1)").arg(QStringLiteral("color|stretch|fit|center|tile|zoom")), tr("MODE"));
     parser.addOption(wallpaperModeOption);
 
-    QCommandLineOption showPrefOption(QStringLiteral("show-pref"), tr("Open Preferences dialog on the page with the specified name"), tr("NAME"));
+    QCommandLineOption showPrefOption(QStringLiteral("show-pref"), tr("Open Preferences dialog on the page with the specified name") + QStringLiteral("\n") + tr("NAME") + QStringLiteral("=(behavior|display|ui|thumbnail|volume|advanced)"), tr("NAME"));
     parser.addOption(showPrefOption);
 
     parser.addPositionalArgument(QStringLiteral("files"), tr("Files or directories to open"), tr("[FILE1, FILE2,...]"));
@@ -372,9 +395,39 @@ void Application::onUserDirsChanged() {
 
 void Application::onAboutToQuit() {
     qDebug("aboutToQuit");
+    // save custom positions of desktop items
+    if(!desktopWindows_.isEmpty()) {
+        desktopWindows_.first()->saveItemPositions();
+    }
+
     settings_.save();
 }
 
+void Application::cleanPerFolderConfig() {
+    // first save the perfolder config cache to have the list of all custom folders
+    Fm::FolderConfig::saveCache();
+    // then remove non-existent native folders from the list of custom folders
+    QByteArray perFolderConfig = (settings_.profileDir(profileName_) + QStringLiteral("/dir-settings.conf"))
+                                 .toLocal8Bit();
+    GKeyFile* kf = g_key_file_new();
+    if(g_key_file_load_from_file(kf, perFolderConfig.constData(), G_KEY_FILE_NONE, nullptr)) {
+        bool removed(false);
+        gchar **groups = g_key_file_get_groups(kf, nullptr);
+        for(int i = 0; groups[i] != nullptr; i++) {
+            const gchar *g = groups[i];
+            if(Fm::FilePath::fromPathStr(g).isNative() && !QDir(QString::fromUtf8(g)).exists()) {
+                g_key_file_remove_group(kf, g, nullptr);
+                removed = true;
+            }
+        }
+        g_strfreev(groups);
+        if(removed) {
+            g_key_file_save_to_file(kf, perFolderConfig.constData(), nullptr);
+        }
+    }
+    g_key_file_free(kf);
+}
+
 /*bool Application::eventFilter(QObject* watched, QEvent* event) {
     if(watched == desktop()) {
         if(event->type() == QEvent::StyleChange ||
@@ -447,7 +500,7 @@ void Application::desktopManager(bool enabled) {
     enableDesktopManager_ = enabled;
 }
 
-void Application::desktopPrefrences(QString page) {
+void Application::desktopPrefrences(const QString& page) {
     // show desktop preference window
     if(!desktopPreferencesDialog_) {
         desktopPreferencesDialog_ = new DesktopPreferencesDialog();
@@ -509,19 +562,20 @@ void Application::connectToServer() {
     dlg->show();
 }
 
-void Application::launchFiles(QString cwd, QStringList paths, bool inNewWindow, bool reopenLastTabs) {
+void Application::launchFiles(const QString& cwd, const QStringList& paths, bool inNewWindow, bool reopenLastTabs) {
     Fm::FilePathList pathList;
     Fm::FilePath cwd_path;
+    auto _paths = paths;
 
     reopenLastTabs = reopenLastTabs && settings_.reopenLastTabs() && !settings_.tabPaths().isEmpty();
     if(reopenLastTabs) {
-        paths = settings_.tabPaths();
-        paths.removeDuplicates();
+        _paths = settings_.tabPaths();
+        _paths.removeDuplicates();
         // forget tab paths with next windows until the last one is closed
         settings_.setTabPaths(QStringList());
     }
 
-    for(const QString& it : qAsConst(paths)) {
+    for(const QString& it : qAsConst(_paths)) {
         QByteArray pathName = it.toLocal8Bit();
         Fm::FilePath path;
         if(pathName == "~") { // special case for home dir
@@ -539,7 +593,7 @@ void Application::launchFiles(QString cwd, QStringList paths, bool inNewWindow,
             }
             path = cwd_path.relativePath(pathName.constData());
         }
-       pathList.push_back(std::move(path));
+        pathList.push_back(std::move(path));
     }
 
     if(!inNewWindow && settings_.singleWindowMode()) {
@@ -575,9 +629,9 @@ void Application::launchFiles(QString cwd, QStringList paths, bool inNewWindow,
             }
         }
         if(!hasWindow) {
-            paths.clear();
-            paths.push_back(QDir::currentPath());
-            launchFiles(QDir::currentPath(), paths, inNewWindow, false);
+            _paths.clear();
+            _paths.push_back(QDir::currentPath());
+            launchFiles(QDir::currentPath(), _paths, inNewWindow, false);
         }
     }
 }
@@ -601,7 +655,7 @@ void Application::openFolderInTerminal(Fm::FilePath path) {
     }
 }
 
-void Application::preferences(QString page) {
+void Application::preferences(const QString& page) {
     // open preference dialog
     if(!preferencesDialog_) {
         preferencesDialog_ = new PreferencesDialog(page);
@@ -614,7 +668,7 @@ void Application::preferences(QString page) {
     preferencesDialog_.data()->activateWindow();
 }
 
-void Application::setWallpaper(QString path, QString modeString) {
+void Application::setWallpaper(const QString& path, const QString& modeString) {
     static const char* valid_wallpaper_modes[] = {"color", "stretch", "fit", "center", "tile"};
     DesktopWindow::WallpaperMode mode = settings_.wallpaperMode();
     bool changed = false;
@@ -657,6 +711,97 @@ void Application::setWallpaper(QString path, QString modeString) {
     }
 }
 
+/* This method receives a list of file:// URIs from DBus and for each URI opens
+ * a tab showing its content.
+ */
+void Application::ShowFolders(const QStringList& uriList, const QString& startupId __attribute__((unused))) {
+    if(!uriList.isEmpty()) {
+        launchFiles(QDir::currentPath(), uriList, false, false);
+    }
+}
+
+/* This method receives a list of file:// URIs from DBus and opens windows
+ * or tabs for each folder, highlighting all listed items within each.
+ */
+void Application::ShowItems(const QStringList& uriList, const QString& startupId __attribute__((unused))) {
+    std::unordered_map<Fm::FilePath, Fm::FilePathList, Fm::FilePathHash> groups;
+    Fm::FilePathList folders; // used only for keeping the original order
+    for(const auto& u : uriList) {
+        if(auto path = Fm::FilePath::fromPathStr(u.toStdString().c_str())) {
+            if(auto parent = path.parent()) {
+                auto paths = groups[parent];
+                if(std::find(paths.cbegin(), paths.cend(), path) == paths.cend()) {
+                    groups[parent].push_back(std::move(path));
+                }
+                // also remember the order of parent folders
+                if(std::find(folders.cbegin(), folders.cend(), parent) == folders.cend()) {
+                    folders.push_back(std::move(parent));
+                }
+            }
+        }
+    }
+
+    if(groups.empty()) {
+        return;
+    }
+
+    PCManFM::MainWindow* window = nullptr;
+    if(settings_.singleWindowMode()) {
+        window = MainWindow::lastActive();
+        if(window == nullptr) {
+            QWidgetList windows = topLevelWidgets();
+            for(int i = 0; i < windows.size(); ++i) {
+                auto win = windows.at(windows.size() - 1 - i);
+                if(win->inherits("PCManFM::MainWindow")) {
+                    window = static_cast<MainWindow*>(win);
+                    break;
+                }
+            }
+        }
+    }
+    if(window == nullptr) {
+        window = new MainWindow();
+    }
+
+    for(const auto& folder : folders) {
+        window->openFolderAndSelectFles(groups[folder]);
+    }
+
+    window->show();
+    window->raise();
+    window->activateWindow();
+}
+
+/* This method receives a list of file:// URIs from DBus and
+ * for each valid URI opens a property dialog showing its information
+ */
+void Application::ShowItemProperties(const QStringList& uriList, const QString& startupId __attribute__((unused))) {
+    // FIXME: Should we add "Fm::FilePropsDialog::showForPath()" to libfm-qt, instead of doing this?
+    Fm::FilePathList paths;
+    for(const auto& u : uriList) {
+        Fm::FilePath path = Fm::FilePath::fromPathStr(u.toStdString().c_str());
+        if(path) {
+            paths.push_back(std::move(path));
+        }
+    }
+    if(paths.empty()) {
+        return;
+    }
+    auto job = new Fm::FileInfoJob{std::move(paths)};
+    job->setAutoDelete(true);
+    connect(job, &Fm::FileInfoJob::finished, this, &Application::onPropJobFinished, Qt::BlockingQueuedConnection);
+    job->runAsync();
+}
+
+void Application::onPropJobFinished() {
+    auto job = static_cast<Fm::FileInfoJob*>(sender());
+    for(auto file: job->files()) {
+        auto dialog = Fm::FilePropsDialog::showForFile(std::move(file));
+        dialog->raise();
+        dialog->activateWindow();
+    }
+}
+
 DesktopWindow* Application::createDesktopWindow(int screenNum) {
     DesktopWindow* window = new DesktopWindow(screenNum);
 
@@ -713,40 +858,7 @@ void Application::editBookmarks() {
 }
 
 void Application::initVolumeManager() {
-    // WARNING: If the user tries to mount a volume before auto-mounting is started —
-    // and that is especially possible with an encrypted volume — a complete freeze might
-    // happen because MountOperation::wait() uses a local QEventLoop. Therefore, we need
-    // to start auto-mounting only after all probable GUI mount operations are finished.
-    QList<Fm::MountOperation*> ops;
-    const auto windows = topLevelWidgets();
-    for(const auto& win : windows) {
-        if(win->inherits("PCManFM::MainWindow")) {
-            MainWindow* mainWindow = static_cast<MainWindow*>(win);
-            ops << mainWindow->pendingMountOperations();
-        }
-    }
-    if(ops.isEmpty()) { // no pending mount operation
-        reallyInitVolumeManager();
-        return;
-    }
-    for(const Fm::MountOperation* op : ops) {
-        connect(op, &QObject::destroyed, this, [this] {
-            const auto windows = topLevelWidgets();
-            for(const auto& win : windows) {
-                if(win->inherits("PCManFM::MainWindow")) {
-                    MainWindow* mainWindow = static_cast<MainWindow*>(win);
-                    if(!mainWindow->pendingMountOperations().isEmpty()) {
-                        return;
-                    }
-                }
-            }
-            // now, there is no pending mount operation
-            reallyInitVolumeManager();
-        });
-    }
-}
 
-void Application::reallyInitVolumeManager() {
     g_signal_connect(volumeMonitor_, "volume-added", G_CALLBACK(onVolumeAdded), this);
 
     if(settings_.mountOnStartup()) {
