@@ -50,7 +50,7 @@ class ProxyStyle: public QProxyStyle {
 public:
     ProxyStyle() : QProxyStyle() {}
     virtual ~ProxyStyle() {}
-    virtual int styleHint(StyleHint hint, const QStyleOption* option = 0, const QWidget* widget = 0, QStyleHintReturn* returnData = 0) const;
+    virtual int styleHint(StyleHint hint, const QStyleOption* option = nullptr, const QWidget* widget = nullptr, QStyleHintReturn* returnData = nullptr) const;
 };
 
 class Application : public QApplication {
@@ -72,14 +72,21 @@ public:
         return libFm_;
     }
 
+    bool isX11() const {
+      return isX11_;
+    }
+
     // public interface exported via dbus
-    void launchFiles(QString cwd, QStringList paths, bool inNewWindow, bool reopenLastTabs);
-    void setWallpaper(QString path, QString modeString);
-    void preferences(QString page);
-    void desktopPrefrences(QString page);
+    void launchFiles(const QString& cwd, const QStringList& paths, bool inNewWindow, bool reopenLastTabs);
+    void setWallpaper(const QString& path, const QString& modeString);
+    void preferences(const QString& page);
+    void desktopPrefrences(const QString& page);
     void editBookmarks();
     void desktopManager(bool enabled);
     void findFiles(QStringList paths = QStringList());
+    void ShowFolders(const QStringList& uriList, const QString& startupId);
+    void ShowItems(const QStringList& uriList, const QString& startupId);
+    void ShowItemProperties(const QStringList& uriList, const QString& startupId);
     void connectToServer();
 
     bool desktopManagerEnabled() {
@@ -96,6 +103,8 @@ public:
         return profileName_;
     }
 
+    void cleanPerFolderConfig();
+
 protected Q_SLOTS:
     void onAboutToQuit();
     void onSigtermNotified();
@@ -124,11 +133,11 @@ protected:
 
 private Q_SLOTS:
     void onUserDirsChanged();
+    void onPropJobFinished();
 
 private:
     void initWatch();
     void installSigtermHandler();
-    void reallyInitVolumeManager();
 
     bool isPrimaryInstance;
     Fm::LibFmQt libFm_;
@@ -149,6 +158,8 @@ private:
     QString userDesktopFolder_;
     bool lxqtRunning_;
 
+    bool isX11_;
+
     int argc_;
     char** argv_;
 };
