@@ -59,11 +59,7 @@ void DesktopEntryDialog::onClickingIconButton() {
                                                           tr("Images (*.png *.xpm *.svg *.svgz )"));
     if(!iconPath.isEmpty()) {
         if(iconPath.startsWith(iconDir)) { // a theme icon
-#if (QT_VERSION >= QT_VERSION_CHECK(5,15,0))
             QStringList parts = iconPath.split(QStringLiteral("/"), Qt::SkipEmptyParts);
-#else
-            QStringList parts = iconPath.split(QStringLiteral("/"), QString::SkipEmptyParts);
-#endif
             if(!parts.isEmpty()) {
                 QString iconName = parts.at(parts.count() - 1);
                 int ln = iconName.lastIndexOf(QLatin1String("."));
@@ -136,6 +132,7 @@ void DesktopEntryDialog::accept() {
     // make file name from entry name but so that it doesn't exist on Desktop
     name = name.simplified();
     name.replace(QChar(QChar::Space), QLatin1Char('_'));
+    name.replace(QLatin1Char('/'), QLatin1Char('_'));
     QString suffix;
     int i = 0;
     while(QFile::exists(QString::fromUtf8(pathStrPtr.get())
