@@ -131,6 +131,8 @@ DesktopPreferencesDialog::DesktopPreferencesDialog(QWidget* parent, Qt::WindowFl
 
   ui.defaultFileManager->setChecked(settings.openWithDefaultFileManager());
 
+  ui.allSticky->setChecked(settings.allSticky());
+
   resize(sizeHint()); // show it compact
 }
 
@@ -208,6 +210,8 @@ void DesktopPreferencesDialog::applySettings()
 
   settings.setOpenWithDefaultFileManager(ui.defaultFileManager->isChecked());
 
+  settings.setAllSticky(ui.allSticky->isChecked());
+
   settings.save();
 }
 
@@ -327,7 +331,7 @@ void DesktopPreferencesDialog::onBrowseDesktopFolderClicked()
   }
 }
 
-void DesktopPreferencesDialog::selectPage(QString name) {
+void DesktopPreferencesDialog::selectPage(const QString& name) {
   QWidget* page = findChild<QWidget*>(name + QStringLiteral("Page"));
   if(page)
     ui.tabWidget->setCurrentWidget(page);
