@@ -137,6 +137,8 @@ DesktopWindow::DesktopWindow(int screenNum):
         connect(proxyModel_, &Fm::ProxyFolderModel::rowsAboutToBeRemoved, this, &DesktopWindow::onRowsAboutToBeRemoved);
         connect(proxyModel_, &Fm::ProxyFolderModel::layoutChanged, this, &DesktopWindow::onLayoutChanged);
         connect(proxyModel_, &Fm::ProxyFolderModel::sortFilterChanged, this, &DesktopWindow::onModelSortFilterChanged);
+
+        connect(this, &Fm::FolderView::inlineRenamed, this, &DesktopWindow::onInlineRenaming);
     }
 
     // remove frame
@@ -391,6 +393,9 @@ void DesktopWindow::onTrashChanged(GFileMonitor* /*monitor*/, GFile* /*gf*/, GFi
 }
 
 void DesktopWindow::updateTrashIcon() {
+    if(listView_->isHidden()) {
+        return; // will be updated as soon as dekstop is shown
+    }
     struct UpdateTrashData {
         QPointer<DesktopWindow> desktop;
         Fm::FilePath trashPath;
@@ -506,7 +511,7 @@ void DesktopWindow::setDesktopFolder() {
     }
 }
 
-void DesktopWindow::setWallpaperFile(QString filename) {
+void DesktopWindow::setWallpaperFile(const QString& filename) {
     wallpaperFile_ = filename;
 }
 
@@ -514,11 +519,11 @@ void DesktopWindow::setWallpaperMode(WallpaperMode mode) {
     wallpaperMode_ = mode;
 }
 
-void DesktopWindow::setLastSlide(QString filename) {
+void DesktopWindow::setLastSlide(const QString& filename) {
     lastSlide_ = filename;
 }
 
-void DesktopWindow::setWallpaperDir(QString dirname) {
+void DesktopWindow::setWallpaperDir(const QString& dirname) {
     wallpaperDir_ = dirname;
 }
 
@@ -944,7 +949,7 @@ void DesktopWindow::onFileClicked(int type, const std::shared_ptr<const Fm::File
     }
     else {
         // special right-click menus for our desktop shortcuts
-        if(fileInfo && fileInfo->isDesktopEntry() && type == Fm::FolderView::ContextMenuClick) {
+        if(fileInfo && fileInfo->isDesktopEntry() && type == Fm::FolderView::ContextMenuClick && hasSelection()) {
             Settings& settings = static_cast<Application* >(qApp)->settings();
             const QStringList ds = settings.desktopShortcuts();
             if(!ds.isEmpty()) {
@@ -960,10 +965,12 @@ void DesktopWindow::onFileClicked(int type, const std::shared_ptr<const Fm::File
                         onFileClicked(Fm::FolderView::ActivatedClick, fileInfo);
                     });
                     // "Stick" action for all
-                    action = menu->addAction(tr("Stic&k to Current Position"));
-                    action->setCheckable(true);
-                    action->setChecked(customItemPos_.find(fileInfo->name()) != customItemPos_.cend());
-                    connect(action, &QAction::toggled, this, &DesktopWindow::onStickToCurrentPos);
+                    if(!settings.allSticky()) {
+                        action = menu->addAction(tr("Stic&k to Current Position"));
+                        action->setCheckable(true);
+                        action->setChecked(customItemPos_.find(fileInfo->name()) != customItemPos_.cend());
+                        connect(action, &QAction::toggled, this, &DesktopWindow::onStickToCurrentPos);
+                    }
                     // "Empty Trash" action for Trash shortcut
                     if(fileName == QLatin1String("trash-can.desktop")) {
                         menu->addSeparator();
@@ -997,21 +1004,24 @@ void DesktopWindow::onFileClicked(int type, const std::shared_ptr<const Fm::File
 void DesktopWindow::prepareFileMenu(Fm::FileMenu* menu) {
     // qDebug("DesktopWindow::prepareFileMenu");
     PCManFM::View::prepareFileMenu(menu);
-    QAction* action = new QAction(tr("Stic&k to Current Position"), menu);
-    action->setCheckable(true);
-    menu->insertSeparator(menu->separator2());
-    menu->insertAction(menu->separator2(), action);
 
-    bool checked(true);
-    auto files = menu->files();
-    for(const auto& file : files) {
-        if(customItemPos_.find(file->name()) == customItemPos_.cend()) {
-            checked = false;
-            break;
+    if(!static_cast<Application*>(qApp)->settings().allSticky()) {
+        QAction* action = new QAction(tr("Stic&k to Current Position"), menu);
+        action->setCheckable(true);
+        menu->insertSeparator(menu->separator2());
+        menu->insertAction(menu->separator2(), action);
+
+        bool checked(true);
+        auto files = menu->files();
+        for(const auto& file : files) {
+            if(customItemPos_.find(file->name()) == customItemPos_.cend()) {
+                checked = false;
+                break;
+            }
         }
+        action->setChecked(checked);
+        connect(action, &QAction::toggled, this, &DesktopWindow::onStickToCurrentPos);
     }
-    action->setChecked(checked);
-    connect(action, &QAction::toggled, this, &DesktopWindow::onStickToCurrentPos);
 }
 
 void DesktopWindow::prepareFolderMenu(Fm::FolderMenu* menu) {
@@ -1044,7 +1054,11 @@ void DesktopWindow::toggleDesktop() {
     // a relayout is needed on showing the items for the first time
     // because the positions aren't updated while the view is hidden
     if(!desktopHideItems_) {
+        listView_->setUpdatesEnabled(false); // prevent items from shaking
         listView_->setFocus(); // refocus the view
+        if(trashMonitor_) {
+            updateTrashIcon();
+        }
         queueRelayout();
     }
     else {
@@ -1105,7 +1119,7 @@ void DesktopWindow::onRowsAboutToBeRemoved(const QModelIndex& parent, int start,
             }
         }
         if(changed) {
-            saveItemPositions();
+            storeCustomPos();
         }
     }
     listView_->setUpdatesEnabled(false);
@@ -1141,13 +1155,13 @@ void DesktopWindow::onFolderFinishLoading() {
 void DesktopWindow::onFilesAdded(const Fm::FileInfoList files) {
     if(static_cast<Application*>(qApp)->settings().selectNewFiles()) {
         if(!selectionTimer_) {
-            selectFiles(files, false);
             selectionTimer_ = new QTimer (this);
             selectionTimer_->setSingleShot(true);
-            selectionTimer_->start(200);
+            if(selectFiles(files, false)) {
+                selectionTimer_->start(200);
+            }
         }
-        else {
-            selectFiles(files, selectionTimer_->isActive());
+        else if(selectFiles(files, selectionTimer_->isActive())) {
             selectionTimer_->start(200);
         }
     }
@@ -1203,6 +1217,20 @@ void DesktopWindow::removeBottomGap() {
     setShadow(settings.desktopShadowColor());
 }
 
+void DesktopWindow::onInlineRenaming(const QString& oldName, const QString& newName) {
+    // preserve custom position on inline renaming
+    auto old_name = oldName.toStdString();
+    for(auto it = customItemPos_.cbegin(); it != customItemPos_.cend(); ++it) {
+        if(it->first == old_name) {
+            auto pos = it->second;
+            customItemPos_.erase(it);
+            customItemPos_[newName.toStdString()] = pos;
+            storeCustomPos();
+            break;
+        }
+    }
+}
+
 void DesktopWindow::paintBackground(QPaintEvent* event) {
     // This is to workaround Qt bug 54384 which affects Qt >= 5.6
     // https://bugreports.qt.io/browse/QTBUG-54384
@@ -1248,20 +1276,31 @@ void DesktopWindow::trustOurDesktopShortcut(std::shared_ptr<const Fm::FileInfo>
     }
 }
 
+QRect DesktopWindow::getWorkArea(QScreen* screen) const {
+    QRect workArea = screen->availableVirtualGeometry();
+    workArea.adjust(WORK_AREA_MARGIN, WORK_AREA_MARGIN, -WORK_AREA_MARGIN, -WORK_AREA_MARGIN);
+    // switch between right and left with RTL to use the usual (LTR) calculations later
+    if(layoutDirection() == Qt::RightToLeft) {
+        workArea.moveLeft(rect().right() - workArea.right());
+    }
+    return workArea;
+}
+
 // QListView does item layout in a very inflexible way, so let's do our custom layout again.
 // FIXME: this is very inefficient, but due to the design flaw of QListView, this is currently the only workaround.
 void DesktopWindow::relayoutItems() {
-    auto screen = getDesktopScreen();
-    if(screen == nullptr) {
-        return;
-    }
-    loadItemPositions(); // something may have changed
-    // qDebug("relayoutItems()");
     if(relayoutTimer_) {
         // this slot might be called from the timer, so we cannot delete it directly here.
         relayoutTimer_->deleteLater();
         relayoutTimer_ = nullptr;
     }
+    auto screen = getDesktopScreen();
+    if(screen == nullptr) {
+        return;
+    }
+    retrieveCustomPos(); // something may have changed
+
+    bool allSticky = static_cast<Application*>(qApp)->settings().allSticky();
 
     int row = 0;
     int rowCount = proxyModel_->rowCount();
@@ -1269,15 +1308,17 @@ void DesktopWindow::relayoutItems() {
     auto delegate = static_cast<Fm::FolderItemDelegate*>(listView_->itemDelegateForColumn(0));
     auto itemSize = delegate->itemSize();
 
-    QRect workArea = screen->availableVirtualGeometry();
-    workArea.adjust(WORK_AREA_MARGIN, WORK_AREA_MARGIN, -WORK_AREA_MARGIN, -WORK_AREA_MARGIN);
-    // qDebug() << "workArea" << screenNum_ <<  workArea;
+    QRect workArea = getWorkArea(screen);
+
     // FIXME: we use an internal class declared in a private header here, which is pretty bad.
     QPoint pos = workArea.topLeft();
     for(; row < rowCount; ++row) {
         QModelIndex index = proxyModel_->index(row, 0);
         int itemWidth = delegate->sizeHint(listView_->getViewOptions(), index).width();
         auto file = proxyModel_->fileInfoFromIndex(index);
+        if(file == nullptr) {
+            continue;
+        }
         auto name = file->name();
         if(filesToTrust_.contains(QString::fromStdString(name))) {
             file->setTrustable(true);
@@ -1285,13 +1326,12 @@ void DesktopWindow::relayoutItems() {
         }
         else if(file->isDesktopEntry()) {
             trustOurDesktopShortcut(file);
-         }
+        }
         auto find_it = customItemPos_.find(name);
         if(find_it != customItemPos_.cend()) { // the item has a custom position
             QPoint customPos = find_it->second;
-            // center the contents vertically
+            // center the contents horizontally
             listView_->setPositionForIndex(customPos + QPoint((itemSize.width() - itemWidth) / 2, 0), index);
-            // qDebug() << "set custom pos:" << name << row << index << customPos;
             continue;
         }
         // check if the current pos is already occupied by a custom item
@@ -1307,9 +1347,14 @@ void DesktopWindow::relayoutItems() {
             --row;
         }
         else {
-            // center the contents vertically
+            // center the contents horizontally
             listView_->setPositionForIndex(pos + QPoint((itemSize.width() - itemWidth) / 2, 0), index);
-            // qDebug() << "set pos" << name << row << index << pos;
+
+            // if all items should be sticky, add this item to custom postions
+            if(allSticky && pos.x() + itemSize.width() <= workArea.right() + 1) {
+                customItemPos_[name] = pos;
+                customPosStorage_[name] = pos;
+            }
         }
         // move to next cell in the column
         pos.setY(pos.y() + itemSize.height() + listView_->spacing());
@@ -1344,70 +1389,81 @@ void DesktopWindow::relayoutItems() {
     }
 }
 
+// NOTE: We load custom positions from the config file at startup and store them in
+// customPosStorage_ to keep track of them without constantly saving them to the disk
+// during the session. customPosStorage_ should be written to the disk on exiting.
+//
+// In contrast, customItemPos_ reflects the real custom positions at the moment, which
+// may not be suitable for saving to the disk because it may be affected by temporary
+// changes to geometry that we do not want to remember at the next session.
+
 void DesktopWindow::loadItemPositions() {
-    auto screen = getDesktopScreen();
-    if(screen == nullptr) {
-        return;
-    }
-    // load custom item positions
-    customItemPos_.clear();
+    // load custom item positions from the config file and store them in the memory
+    customPosStorage_.clear();
     Settings& settings = static_cast<Application*>(qApp)->settings();
     QString configFile = QStringLiteral("%1/desktop-items-%2.conf").arg(settings.profileDir(settings.profileName())).arg(screenNum_);
     QSettings file(configFile, QSettings::IniFormat);
 
-    auto delegate = static_cast<Fm::FolderItemDelegate*>(listView_->itemDelegateForColumn(0));
-    auto grid = delegate->itemSize();
-    QRect workArea = screen->availableVirtualGeometry();
-    workArea.adjust(WORK_AREA_MARGIN, WORK_AREA_MARGIN, -WORK_AREA_MARGIN, -WORK_AREA_MARGIN);
-    QString desktopDir = QStandardPaths::writableLocation(QStandardPaths::DesktopLocation);
-    desktopDir += QLatin1Char('/');
-    std::vector<QPoint> usedPos;
-    for(auto& item: customItemPos_) {
-        usedPos.push_back(item.second);
-    }
-
-    // FIXME: this is inefficient
     const auto names = file.childGroups();
     for(const QString& name : names) {
-        if(!QFile::exists(desktopDir + name)) {
-            // the file may have been removed from outside LXQT
-            continue;
-        }
         file.beginGroup(name);
-        QVariant var = file.value(QStringLiteral("pos"));
-        if(var.isValid()) {
-            QPoint customPos = var.toPoint();
-            if(customPos.x() >= workArea.x() && customPos.y() >= workArea.y()
-                    && customPos.x() + grid.width() <= workArea.right() + 1
-                    && customPos.y() + grid.height() <= workArea.bottom() + 1) {
-                // correct positions that are't aligned to the grid
-                alignToGrid(customPos, workArea.topLeft(), grid, listView_->spacing());
-                // FIXME: this is very inefficient
-                while(std::find(usedPos.cbegin(), usedPos.cend(), customPos) != usedPos.cend()) {
-                    customPos.setY(customPos.y() + grid.height() + listView_->spacing());
-                    if(customPos.y() + grid.height() > workArea.bottom() + 1) {
-                        customPos.setX(customPos.x() + grid.width() + listView_->spacing());
-                        customPos.setY(workArea.top());
-                    }
-                }
-                customItemPos_[name.toStdString()] = customPos;
-                usedPos.push_back(customPos);
-            }
-        }
+        customPosStorage_[name.toStdString()] = file.value(QStringLiteral("pos")).toPoint();
         file.endGroup();
     }
 }
 
+void DesktopWindow::retrieveCustomPos() {
+    // retrieve custom item positions from the memory and normalize them
+    auto screen = getDesktopScreen();
+    if(screen == nullptr) {
+        return;
+    }
+    customItemPos_.clear();
+    auto delegate = static_cast<Fm::FolderItemDelegate*>(listView_->itemDelegateForColumn(0));
+    auto grid = delegate->itemSize();
+    QRect workArea = getWorkArea(screen);
+    QString desktopDir = QStandardPaths::writableLocation(QStandardPaths::DesktopLocation);
+    desktopDir += QLatin1Char('/');
+    std::vector<QPoint> usedPos;
+
+    for(auto it = customPosStorage_.cbegin(); it != customPosStorage_.cend(); ++it) {
+        auto& name = it->first;
+        if(!QFile::exists(desktopDir + QString::fromStdString(name))) {
+            continue;
+        }
+        auto customPos = it->second;
+        // skip positions before the left edge, above the top edge or
+        // below the bottom edge of the work area
+        if(customPos.x() >= workArea.left()
+           && customPos.y() >= workArea.top()
+           && customPos.y() + grid.height() <= workArea.bottom() + 1) {
+            // correct positions that aren't aligned to the grid
+            alignToGrid(customPos, workArea.topLeft(), grid, listView_->spacing());
+            // guarantee different positions for different items
+            while(std::find(usedPos.cbegin(), usedPos.cend(), customPos) != usedPos.cend()) {
+                customPos.setY(customPos.y() + grid.height() + listView_->spacing());
+                if(customPos.y() + grid.height() > workArea.bottom() + 1) {
+                    customPos.setX(customPos.x() + grid.width() + listView_->spacing());
+                    customPos.setY(workArea.top());
+                }
+            }
+            // also, skip positions after the right edge of the work area
+            if(customPos.x() + grid.width() <= workArea.right() + 1) {
+                customItemPos_[name] = customPos;
+                usedPos.push_back(customPos);
+            }
+        }
+    }
+}
+
 void DesktopWindow::saveItemPositions() {
+    // write custom item positions to the config file
     Settings& settings = static_cast<Application*>(qApp)->settings();
-    // store custom item positions
     QString configFile = QStringLiteral("%1/desktop-items-%2.conf").arg(settings.profileDir(settings.profileName())).arg(screenNum_);
-    // FIXME: using QSettings here is inefficient and it's not friendly to UTF-8.
     QSettings file(configFile, QSettings::IniFormat);
     file.clear(); // remove all existing entries
 
-    // FIXME: we have to remove dead entries not associated to any files?
-    for(auto it = customItemPos_.cbegin(); it != customItemPos_.cend(); ++it) {
+    for(auto it = customPosStorage_.cbegin(); it != customPosStorage_.cend(); ++it) {
         auto& name = it->first;
         auto& pos = it->second;
         file.beginGroup(QString::fromStdString(name));
@@ -1416,27 +1472,34 @@ void DesktopWindow::saveItemPositions() {
     }
 }
 
+void DesktopWindow::storeCustomPos() {
+    // store custom item positions in the memory
+    customPosStorage_.clear();
+    customPosStorage_ = customItemPos_;
+}
+
 void DesktopWindow::onStickToCurrentPos(bool toggled) {
     QModelIndexList indexes = listView_->selectionModel()->selectedIndexes();
     if(!indexes.isEmpty()) {
         bool relayout(false);
         QModelIndexList::const_iterator it;
         for(it = indexes.constBegin(); it != indexes.constEnd(); ++it) {
-            auto file = proxyModel_->fileInfoFromIndex(*it);
-            auto name = file->name();
-            if(toggled) { // remember the current custom position
-                QRect itemRect = listView_->rectForIndex(*it);
-                customItemPos_[name] = itemRect.topLeft();
-            }
-            else { // cancel custom position and perform relayout
-                auto item = customItemPos_.find(name);
-                if(item != customItemPos_.end()) {
-                    customItemPos_.erase(item);
-                    relayout = true;
+            if(auto file = proxyModel_->fileInfoFromIndex(*it)) {
+                auto name = file->name();
+                if(toggled) { // remember the current custom position
+                    QRect itemRect = listView_->rectForIndex(*it);
+                    customItemPos_[name] = itemRect.topLeft();
+                }
+                else { // cancel custom position and perform relayout
+                    auto item = customItemPos_.find(name);
+                    if(item != customItemPos_.end()) {
+                        customItemPos_.erase(item);
+                        relayout = true;
+                    }
                 }
             }
         }
-        saveItemPositions();
+        storeCustomPos();
         if(relayout) {
             relayoutItems();
         }
@@ -1620,37 +1683,59 @@ QModelIndex DesktopWindow::navigateWithKey(int key, Qt::KeyboardModifiers modifi
         }
         auto delegate = static_cast<Fm::FolderItemDelegate*>(listView_->itemDelegateForColumn(0));
         auto itemSize = delegate->itemSize();
-        QRect workArea = screen->availableVirtualGeometry();
-        workArea.adjust(WORK_AREA_MARGIN, WORK_AREA_MARGIN, -WORK_AREA_MARGIN, -WORK_AREA_MARGIN);
+        QRect workArea = getWorkArea(screen);
         int columns = workArea.width() / (itemSize.width() + listView_->spacing());
         int rows = workArea.height() / (itemSize.height() + listView_->spacing());
+        bool rtl(layoutDirection() == Qt::RightToLeft);
         while(!index.isValid() && workArea.contains(pos)) {
             switch(key) {
             case Qt::Key_Up:
                 pos.setY(pos.y() - itemSize.height() - listView_->spacing());
                 if(pos.y() < workArea.top()) {
-                    pos.setX(pos.x() - itemSize.width() - listView_->spacing());
+                    if(rtl) {
+                        pos.setX(pos.x() + itemSize.width() + listView_->spacing());
+                    }
+                    else {
+                        pos.setX(pos.x() - itemSize.width() - listView_->spacing());
+                    }
                     pos.setY(workArea.top() + (rows - 1) * (itemSize.height() + listView_->spacing()));
                 }
                 break;
             case Qt::Key_Right:
                 pos.setX(pos.x() + itemSize.width() + listView_->spacing());
                 if(pos.x() + itemSize.width() > workArea.right() + 1) {
-                    pos.setY(pos.y() + itemSize.height() + listView_->spacing());
-                    pos.setX(workArea.left());
+                    if(rtl) {
+                        pos.setY(pos.y() - itemSize.height() - listView_->spacing());
+                        pos.setX(workArea.right() + 1 - (columns - 1) * (itemSize.width() + listView_->spacing()));
+                    }
+                    else {
+                        pos.setY(pos.y() + itemSize.height() + listView_->spacing());
+                        pos.setX(workArea.left());
+                    }
                 }
                 break;
             case Qt::Key_Left:
                 pos.setX(pos.x() - itemSize.width() - listView_->spacing());
                 if(pos.x() < workArea.left()) {
-                    pos.setY(pos.y() - itemSize.height() - listView_->spacing());
-                    pos.setX(workArea.left() + (columns - 1) * (itemSize.width() + listView_->spacing()));
+                    if(rtl) {
+                        pos.setY(pos.y() + itemSize.height() + listView_->spacing());
+                        pos.setX(workArea.right() + 1 - itemSize.width() - listView_->spacing());
+                    }
+                    else {
+                        pos.setY(pos.y() - itemSize.height() - listView_->spacing());
+                        pos.setX(workArea.left() + (columns - 1) * (itemSize.width() + listView_->spacing()));
+                    }
                 }
                 break;
             default: // consider any other value as Qt::Key_Down
                 pos.setY(pos.y() + itemSize.height() + listView_->spacing());
                 if(pos.y() + itemSize.height() > workArea.bottom() + 1) {
-                    pos.setX(pos.x() + itemSize.width() + listView_->spacing());
+                    if(rtl) {
+                        pos.setX(pos.x() - itemSize.width() - listView_->spacing());
+                    }
+                    else {
+                        pos.setX(pos.x() + itemSize.width() + listView_->spacing());
+                    }
                     pos.setY(workArea.top());
                 }
                 break;
@@ -1769,8 +1854,7 @@ void DesktopWindow::childDragMoveEvent(QDragMoveEvent* e) { // see DesktopWindow
     }
     auto delegate = static_cast<Fm::FolderItemDelegate*>(listView_->itemDelegateForColumn(0));
     auto grid = delegate->itemSize();
-    QRect workArea = screen->availableVirtualGeometry();
-    workArea.adjust(WORK_AREA_MARGIN, WORK_AREA_MARGIN, -WORK_AREA_MARGIN, -WORK_AREA_MARGIN);
+    QRect workArea = getWorkArea(screen);
     bool isTrash;
     QRect oldDropRect = dropRect_;
     dropRect_ = QRect();
@@ -1819,9 +1903,7 @@ void DesktopWindow::childDropEvent(QDropEvent* e) {
     }
     auto delegate = static_cast<Fm::FolderItemDelegate*>(listView_->itemDelegateForColumn(0));
     auto grid = delegate->itemSize();
-    QRect workArea = screen->availableVirtualGeometry();
-    workArea.adjust(WORK_AREA_MARGIN, WORK_AREA_MARGIN, -WORK_AREA_MARGIN, -WORK_AREA_MARGIN);
-
+    QRect workArea = getWorkArea(screen);
     const QMimeData* mimeData = e->mimeData();
     bool moveItem = false;
     QModelIndex curIndx = listView_->currentIndex();
@@ -1860,10 +1942,12 @@ void DesktopWindow::childDropEvent(QDropEvent* e) {
             }
         }
     }
+    bool rtl(layoutDirection() == Qt::RightToLeft);
     if(moveItem) {
         e->accept();
         // move selected items to the drop position, preserving their relative positions
-        const QPoint dropPos = e->pos();
+        QPoint dropPos = e->pos();
+
         if(curIndx.isValid()) {
             QPoint curPoint = listView_->visualRect(curIndx).topLeft();
 
@@ -1871,6 +1955,12 @@ void DesktopWindow::childDropEvent(QDropEvent* e) {
             auto file = proxyModel_->fileInfoFromIndex(curIndx);
             if(file) {
                 QPoint pos = dropPos;
+                // NOTE: DND always reports the drop position in the usual (LTR) coordinates.
+                // Therefore, to have the same calculations regardless of the layout direction,
+                // we reverse the x-coordinate with RTL.
+                if(rtl) {
+                    pos.setX(width() - pos.x());
+                }
                 stickToPosition(file->name(), pos, workArea, grid);
             }
 
@@ -1883,13 +1973,16 @@ void DesktopWindow::childDropEvent(QDropEvent* e) {
                 file = proxyModel_->fileInfoFromIndex(indx);
                 if(file) {
                     QPoint nxtDropPos = dropPos + listView_->visualRect(indx).topLeft() - curPoint;
+                    if(rtl) { // like above
+                        nxtDropPos.setX(width() - nxtDropPos.x());
+                    }
                     nxtDropPos.setX(qBound(workArea.left(), nxtDropPos.x(), workArea.right() + 1));
                     nxtDropPos.setY(qBound(workArea.top(), nxtDropPos.y(), workArea.bottom() + 1));
                     stickToPosition(file->name(), nxtDropPos, workArea, grid);
                 }
             }
         }
-        saveItemPositions();
+        storeCustomPos();
         queueRelayout();
     }
     else {
@@ -1926,6 +2019,13 @@ void DesktopWindow::childDropEvent(QDropEvent* e) {
         dropRect_ = QRect();
         listView_->viewport()->update();
 
+        // do not proceed if items are dropped on a directory
+        if(auto file = proxyModel_->fileInfoFromIndex(dropIndex)) {
+            if(file->isDir()) {
+                return;
+            }
+        }
+
         // position dropped items successively, starting with the drop rectangle
         if(mimeData->hasUrls()
            && (e->dropAction() == Qt::CopyAction
@@ -1933,6 +2033,9 @@ void DesktopWindow::childDropEvent(QDropEvent* e) {
                || e->dropAction() == Qt::LinkAction)) {
             const QString desktopDir = XdgDir::readDesktopDir() + QString(QLatin1String("/"));
             QPoint dropPos = e->pos();
+            if(rtl) { // see the previous case for the reason
+                dropPos.setX(width() - dropPos.x());
+            }
             const QList<QUrl> urlList = mimeData->urls();
             bool reachedLastCell = false;
             for(const QUrl& url : urlList) {
@@ -1943,7 +2046,7 @@ void DesktopWindow::childDropEvent(QDropEvent* e) {
                     reachedLastCell = stickToPosition(name.toStdString(), dropPos, workArea, grid, reachedLastCell);
                 }
             }
-            saveItemPositions();
+            storeCustomPos();
         }
     }
 }
@@ -2054,6 +2157,14 @@ void DesktopWindow::alignToGrid(QPoint& pos, const QPoint& topLeft, const QSize&
 QModelIndex DesktopWindow::indexForPos(bool* isTrash, const QPoint& pos, const QRect& workArea, const QSize& grid) const {
     // first normalize the position
     QPoint p(pos);
+
+    // QAbstractItemView::indexAt() always refers to the usual (LTR) coordinates, which is
+    // provided by "pos". Therefore, to have the same normalizing calculations regardless of
+    // the layout direction, we reverse the x-coordinate with RTL and restore it in the end.
+    if(layoutDirection() == Qt::RightToLeft) {
+        p.setX(width() - p.x());
+    }
+
     if(p.y() + grid.height() > workArea.bottom() + 1) {
         p.setY(workArea.bottom() + 1 - grid.height());
     }
@@ -2067,6 +2178,12 @@ QModelIndex DesktopWindow::indexForPos(bool* isTrash, const QPoint& pos, const Q
     // (if there is any item, its icon is immediately below the middle of its top side)
     p.setX(p.x() + (grid.width() + listView_->spacing()) / 2);
     p.setY(p.y() + listView_->spacing() / 2 + getMargins().height() + 1);
+
+    // restore the x-coordinate with RTL
+    if(layoutDirection() == Qt::RightToLeft) {
+        p.setX(width() - p.x());
+    }
+
     // if this is the Trash cell, return its index
     QModelIndex indx = listView_->indexAt(p);
     if(indx.isValid() && indx.model()) {
@@ -2095,6 +2212,7 @@ void DesktopWindow::paintEvent(QPaintEvent* event) {
 void DesktopWindow::setScreenNum(int num) {
     if(screenNum_ != num) {
         screenNum_ = num;
+        loadItemPositions(); // the config file is different
         queueRelayout();
     }
 }
