@@ -64,10 +64,10 @@ public:
     void setShadow(const QColor& color);
     void setBackground(const QColor& color);
     void setDesktopFolder();
-    void setWallpaperFile(QString filename);
+    void setWallpaperFile(const QString& filename);
     void setWallpaperMode(WallpaperMode mode = WallpaperStretch);
-    void setLastSlide(QString filename);
-    void setWallpaperDir(QString dirname);
+    void setLastSlide(const QString& filename);
+    void setWallpaperDir(const QString& dirname);
     void setSlideShowInterval(int interval);
     void setWallpaperRandomize(bool randomize);
 
@@ -79,6 +79,8 @@ public:
 
     void queueRelayout(int delay = 0);
 
+    void saveItemPositions();
+
     int screenNum() const {
         return screenNum_;
     }
@@ -94,7 +96,8 @@ protected:
     virtual void onFileClicked(int type, const std::shared_ptr<const Fm::FileInfo>& fileInfo) override;
 
     void loadItemPositions();
-    void saveItemPositions();
+    void retrieveCustomPos();
+    void storeCustomPos();
 
     QImage loadWallpaperFile(QSize requiredSize);
 
@@ -137,8 +140,11 @@ protected Q_SLOTS:
 
     void updateTrashIcon();
 
+    void onInlineRenaming(const QString& oldName, const QString& newName);
+
 private:
     void removeBottomGap();
+    QRect getWorkArea(QScreen* screen) const;
     void addDesktopActions(QMenu* menu);
     void paintBackground(QPaintEvent* event);
     void paintDropIndicator();
@@ -183,7 +189,8 @@ private:
     bool desktopHideItems_;
 
     int screenNum_;
-    std::unordered_map<std::string, QPoint> customItemPos_;
+    std::unordered_map<std::string, QPoint> customItemPos_; // real custom positions
+    std::unordered_map<std::string, QPoint> customPosStorage_; // savable custom positions
     QTimer* relayoutTimer_;
     QTimer* selectionTimer_;
 
