@@ -23,6 +23,7 @@
 #include <QMenu>
 #include <QMenuBar>
 #include <QAction>
+#include <QWidgetAction>
 #include <QVBoxLayout>
 #include <QMessageBox>
 #include <QSplitter>
@@ -45,6 +46,7 @@
 #include <libfm-qt/pathedit.h>
 #include <libfm-qt/pathbar.h>
 #include <libfm-qt/core/fileinfo.h>
+#include <libfm-qt/mountoperation.h>
 #include "ui_about.h"
 #include "ui_shortcuts.h"
 #include "application.h"
@@ -144,7 +146,7 @@ MainWindow::MainWindow(Fm::FilePath path):
         ui.verticalLayout->setStretch(0, 1);
     }
 
-    splitView_ = settings.splitView();
+    splitView_ = path && settings.splitView(); // splt view needs a path
 
     // hide menu items that are not usable
     //if(!uriExists("computer:///"))
@@ -267,6 +269,25 @@ MainWindow::MainWindow(Fm::FilePath path):
     ui.actionMenu_bar->setChecked(settings.showMenuBar());
     connect(ui.actionMenu_bar, &QAction::triggered, this, &MainWindow::toggleMenuBar);
 
+    // recent files
+    int recentNumber = settings.getRecentFilesNumber();
+    if(recentNumber == 0) {
+        ui.menuRecentFiles->setEnabled(false);
+    }
+    else {
+        QAction* recentAction = nullptr;
+        for(int i = 0; i < recentNumber; ++i) {
+            recentAction = new QAction(this);
+            recentAction->setVisible(false);
+            connect(recentAction, &QAction::triggered, this, &MainWindow::lanunchRecentFile);
+            ui.menuRecentFiles->addAction(recentAction);
+        }
+        ui.menuRecentFiles->addSeparator();
+        ui.menuRecentFiles->addAction(ui.actionClearRecent);
+    }
+    connect(ui.menuRecentFiles, &QMenu::aboutToShow, this, &MainWindow::updateRecenMenu);
+    connect(ui.actionClearRecent, &QAction::triggered, this, &MainWindow::clearRecentMenu);
+
     // create shortcuts
     QShortcut* shortcut;
     shortcut = new QShortcut(QKeySequence(Qt::Key_Escape), this);
@@ -398,16 +419,10 @@ bool MainWindow::eventFilter(QObject* watched, QEvent* event) {
                         // Change the text and base palettes of an inactive view frame a little.
                         // NOTE: Style-sheets aren't used because they can interfere with QStyle.
                         QPalette palette = viewFrame->palette();
-                        QColor txtCol = palette.color(QPalette::Text);
-                        txtCol.setAlphaF(txtCol.alphaF() * 0.7);
-                        palette.setColor(QPalette::Text, txtCol);
-                        palette.setColor(QPalette::WindowText, txtCol); // tabs
-                        // the disabled text color of path-bars shouldn't change because it may be used by arrows
-                        palette.setColor(QPalette::Active, QPalette::ButtonText, txtCol);
-                        palette.setColor(QPalette::Inactive, QPalette::ButtonText, txtCol);
 
                         // There are various ways of getting a distinct color near the base color
                         // but this one gives the best results with almost all palettes:
+                        QColor txtCol = palette.color(QPalette::Text);
                         QColor baseCol = palette.color(QPalette::Base);
                         baseCol.setRgbF(0.9 * baseCol.redF()   + 0.1 * txtCol.redF(),
                                         0.9 * baseCol.greenF() + 0.1 * txtCol.greenF(),
@@ -415,6 +430,22 @@ bool MainWindow::eventFilter(QObject* watched, QEvent* event) {
                                         baseCol.alphaF());
                         palette.setColor(QPalette::Base, baseCol);
 
+                        // view text
+                        txtCol.setAlphaF(txtCol.alphaF() * 0.7);
+                        palette.setColor(QPalette::Text, txtCol);
+
+                        // window text (used in tabs)
+                        txtCol = palette.color(QPalette::WindowText);
+                        txtCol.setAlphaF(txtCol.alphaF() * 0.7);
+                        palette.setColor(QPalette::WindowText, txtCol);
+
+                        // button text (the disabled text color isn't changed because it may be
+                        // used by some styles for drawing disabled path-bar arrow)
+                        txtCol = palette.color(QPalette::ButtonText);
+                        txtCol.setAlphaF(txtCol.alphaF() * 0.7);
+                        palette.setColor(QPalette::Active, QPalette::ButtonText, txtCol);
+                        palette.setColor(QPalette::Inactive, QPalette::ButtonText, txtCol);
+
                         viewFrame->setPalette(palette);
                     }
                 }
@@ -447,9 +478,12 @@ bool MainWindow::eventFilter(QObject* watched, QEvent* event) {
 
 void MainWindow::addViewFrame(const Fm::FilePath& path) {
     ui.actionGo->setVisible(false);
-    Settings& settings = static_cast<Application*>(qApp)->settings();
+    Application* app = static_cast<Application*>(qApp);
+    Settings& settings = app->settings();
     ViewFrame* viewFrame = new ViewFrame();
-    viewFrame->getTabBar()->setDetachable(!splitView_); // no tab DND with the split view
+    // No tab DND with the split view.
+    // WARNING: Wayland has a serious issue related to QDrag that can result in a crash.
+    viewFrame->getTabBar()->setDetachable(!splitView_ && app->isX11());
     viewFrame->getTabBar()->setTabsClosable(settings.showTabClose());
     ui.viewSplitter->addWidget(viewFrame); // the splitter takes ownership of viewFrame
     if(ui.viewSplitter->count() == 1) {
@@ -470,6 +504,11 @@ void MainWindow::addViewFrame(const Fm::FilePath& path) {
     connect(viewFrame->getTabBar(), &QTabBar::tabMoved, this, &MainWindow::onTabBarTabMoved);
     connect(viewFrame->getTabBar(), &QTabBar::tabBarClicked, this, &MainWindow::onTabBarClicked);
     connect(viewFrame->getTabBar(), &QTabBar::customContextMenuRequested, this, &MainWindow::tabContextMenu);
+    connect(viewFrame->getTabBar(), &QTabBar::tabBarDoubleClicked, this, [this](int index) {
+        if(index == -1) {
+            on_actionNewTab_triggered();
+        }
+    });
     connect(viewFrame->getTabBar(), &TabBar::tabDetached, this, &MainWindow::detachTab);
     connect(viewFrame->getStackedWidget(), &QStackedWidget::widgetRemoved, this, &MainWindow::onStackedWidgetWidgetRemoved);
 
@@ -482,7 +521,8 @@ void MainWindow::on_actionSplitView_triggered(bool checked) {
     if(splitView_ == checked) {
         return;
     }
-    Settings& settings = static_cast<Application*>(qApp)->settings();
+    Application* app = static_cast<Application*>(qApp);
+    Settings& settings = app->settings();
     splitView_ = checked;
     settings.setSplitView(splitView_);
     if(splitView_) { // split the view
@@ -539,7 +579,7 @@ void MainWindow::on_actionSplitView_triggered(bool checked) {
         }
 
         // enable tab DND
-        activeViewFrame_->getTabBar()->setDetachable(true);
+        activeViewFrame_->getTabBar()->setDetachable(app->isX11());
         setAcceptDrops(true);
 
         activeViewFrame_->removeTopBar();
@@ -566,6 +606,7 @@ void MainWindow::chdir(Fm::FilePath path, ViewFrame* viewFrame) {
     QTimer::singleShot(0, viewFrame, [this, path, viewFrame] {
         if(TabPage* page = currentPage(viewFrame)) {
             page->chdir(path, true);
+            setTabIcon(page);
             if(viewFrame == activeViewFrame_) {
                 updateUIForCurrentPage();
             }
@@ -680,6 +721,10 @@ int MainWindow::addTabWithPage(TabPage* page, ViewFrame* viewFrame, Fm::FilePath
     if(!settings.alwaysShowTabs()) {
         viewFrame->getTabBar()->setVisible(viewFrame->getTabBar()->count() > 1);
     }
+
+    // also set tab icon (if the folder is customized)
+    setTabIcon(page);
+
     return index;
 }
 
@@ -710,6 +755,61 @@ void MainWindow::toggleMenuBar(bool /*checked*/) {
     settings.setShowMenuBar(showMenuBar);
 }
 
+void MainWindow::updateRecenMenu() {
+    Settings& settings = static_cast<Application*>(qApp)->settings();
+    int recentNumber = settings.getRecentFilesNumber();
+    auto actions = ui.menuRecentFiles->actions();
+    if(actions.size() < recentNumber + 2) { // there is a separator and a clear action
+        return; // not really needed because we guarantee that it doesn't happen
+    }
+    auto recentFiles = settings.getRecentFiles();
+    int recentSize = recentFiles.size();
+    QFontMetrics metrics(ui.menuRecentFiles->font());
+    int w = 150 * metrics.horizontalAdvance(QLatin1Char(' ')); // for eliding long texts
+    for(int i = 0; i < recentNumber; ++i) {
+        if(i < recentSize) {
+            auto fileName = recentFiles.at(i);
+            actions.at(i)->setText(metrics.elidedText(fileName, Qt::ElideMiddle, w));
+            QIcon icon;
+            auto mimeType = Fm::MimeType::guessFromFileName(fileName.toLocal8Bit().constData());
+            if(!mimeType->isUnknownType()) {
+                if(auto icn = mimeType->icon()) {
+                    icon = icn->qicon();
+                }
+            }
+            actions.at(i)->setIcon(icon);
+            actions.at(i)->setData(fileName);
+            actions.at(i)->setVisible(true);
+        }
+        else {
+            actions.at(i)->setText(QString());
+            actions.at(i)->setIcon(QIcon());
+            actions.at(i)->setData(QVariant());
+            actions.at(i)->setVisible(false);
+        }
+    }
+    ui.actionClearRecent->setEnabled(recentSize != 0);
+}
+
+void MainWindow::clearRecentMenu() {
+    Settings& settings = static_cast<Application*>(qApp)->settings();
+    settings.clearRecentFiles();
+    updateRecenMenu();
+}
+
+void MainWindow::lanunchRecentFile() {
+    if(QAction *action = qobject_cast<QAction*>(QObject::sender())) {
+        Settings& settings = static_cast<Application*>(qApp)->settings();
+        auto pathStr = action->data().toString();
+        settings.addRecentFile(pathStr);
+        auto pathArray = pathStr.toLocal8Bit();
+        auto path = Fm::FilePath::fromLocalPath(pathArray.constData());
+        Fm::FilePathList pathList;
+        pathList.push_back(std::move(path));
+        fileLauncher_.launchPaths(nullptr, pathList);
+    }
+}
+
 void MainWindow::onPathEntryReturnPressed() {
     Fm::PathEdit* pathEntry = pathEntry_;
     if(pathEntry == nullptr) {
@@ -759,6 +859,7 @@ void MainWindow::on_actionGoUp_triggered() {
     QTimer::singleShot(0, this, [this] {
         if(TabPage* page = currentPage()) {
             page->up();
+            setTabIcon(page);
             updateUIForCurrentPage();
         }
     });
@@ -768,6 +869,7 @@ void MainWindow::on_actionGoBack_triggered() {
     QTimer::singleShot(0, this, [this] {
         if(TabPage* page = currentPage()) {
             page->backward();
+            setTabIcon(page);
             updateUIForCurrentPage();
         }
     });
@@ -777,6 +879,7 @@ void MainWindow::on_actionGoForward_triggered() {
     QTimer::singleShot(0, this, [this] {
         if(TabPage* page = currentPage()) {
             page->forward();
+            setTabIcon(page);
             updateUIForCurrentPage();
         }
     });
@@ -866,7 +969,7 @@ void MainWindow::on_actionFolderProperties_triggered() {
 
 void MainWindow::on_actionShowHidden_triggered(bool checked) {
     currentPage()->setShowHidden(checked);
-    ui.sidePane->setShowHidden(checked);
+    // visibility of hidden folders in directory tree is toggled by onTabPageSortFilterChanged()
 }
 
 void MainWindow::on_actionShowThumbnails_triggered(bool checked) {
@@ -925,9 +1028,29 @@ void MainWindow::on_actionHiddenLast_triggered(bool checked) {
     currentPage()->setSortHiddenLast(checked);
 }
 
-void MainWindow::on_actionPreserveView_triggered(bool /*checked*/) {
+void MainWindow::on_actionPreserveView_triggered(bool checked) {
     TabPage* page = currentPage();
-    page->setCustomizedView(!page->hasCustomizedView());
+    page->setCustomizedView(checked);
+    if(checked) {
+        ui.actionPreserveViewRecursive->setChecked(false);
+    }
+    ui.actionGoToCustomizedViewSource->setVisible(page->hasInheritedCustomizedView());
+    setTabIcon(page);
+}
+
+void MainWindow::on_actionPreserveViewRecursive_triggered(bool checked) {
+    TabPage* page = currentPage();
+    page->setCustomizedView(checked, true);
+    if(checked) {
+        ui.actionPreserveView->setChecked(false);
+    }
+    ui.actionGoToCustomizedViewSource->setVisible(page->hasInheritedCustomizedView());
+    setTabIcon(page);
+}
+
+void MainWindow::on_actionGoToCustomizedViewSource_triggered() {
+    currentPage()->goToCustomizedViewSource();
+    updateUIForCurrentPage();
 }
 
 void MainWindow::on_actionFilter_triggered(bool checked) {
@@ -1021,11 +1144,7 @@ void MainWindow::on_actionAddToBookmarks_triggered() {
             auto parent = cwd.parent();
             if(!parent.isValid() || parent == cwd) { // a root path
                 bookmarkName = QString::fromUtf8(cwd.displayName().get());
-#if (QT_VERSION >= QT_VERSION_CHECK(5,15,0))
                 auto parts = bookmarkName.split(QLatin1Char('/'), Qt::SkipEmptyParts);
-#else
-                auto parts = bookmarkName.split(QLatin1Char('/'), QString::SkipEmptyParts);
-#endif
                 if(!parts.isEmpty()) {
                     bookmarkName = parts.last();
                 }
@@ -1077,19 +1196,27 @@ void MainWindow::on_actionHiddenShortcuts_triggered() {
 }
 
 void MainWindow::on_actionIconView_triggered() {
-    currentPage()->setViewMode(Fm::FolderView::IconMode);
+    TabPage* page = currentPage();
+    page->setViewMode(Fm::FolderView::IconMode);
+    setTabIcon(page);
 }
 
 void MainWindow::on_actionCompactView_triggered() {
-    currentPage()->setViewMode(Fm::FolderView::CompactMode);
+    TabPage* page = currentPage();
+    page->setViewMode(Fm::FolderView::CompactMode);
+    setTabIcon(page);
 }
 
 void MainWindow::on_actionDetailedList_triggered() {
-    currentPage()->setViewMode(Fm::FolderView::DetailedListMode);
+    TabPage* page = currentPage();
+    page->setViewMode(Fm::FolderView::DetailedListMode);
+    setTabIcon(page);
 }
 
 void MainWindow::on_actionThumbnailView_triggered() {
-    currentPage()->setViewMode(Fm::FolderView::ThumbnailMode);
+    TabPage* page = currentPage();
+    page->setViewMode(Fm::FolderView::ThumbnailMode);
+    setTabIcon(page);
 }
 
 void MainWindow::onTabBarCloseRequested(int index) {
@@ -1124,14 +1251,10 @@ void MainWindow::onTabBarTabMoved(int from, int to) {
     }
 }
 
-QList<Fm::MountOperation*> MainWindow::pendingMountOperations() const {
-    return ui.sidePane->findChildren<MountOperation*>();
-}
-
 void MainWindow::onFolderUnmounted() {
     TabPage* tabPage = static_cast<TabPage*>(sender());
     if(ViewFrame* viewFrame = viewFrameForTabPage(tabPage)) {
-        const QList<MountOperation*> ops = pendingMountOperations();
+        const QList<MountOperation*> ops = ui.sidePane->findChildren<MountOperation*>();
         if(ops.isEmpty()) { // unmounting is done somewhere else
             Settings& settings = static_cast<Application*>(qApp)->settings();
             if(settings.closeOnUnmount()) {
@@ -1142,21 +1265,23 @@ void MainWindow::onFolderUnmounted() {
                 tabPage->deleteLater();
             }
             else {
-                tabPage->chdir(Fm::FilePath::homeDir(), viewFrame);
+                tabPage->chdir(Fm::FilePath::homeDir(), true);
+                setTabIcon(tabPage);
                 updateUIForCurrentPage();
             }
         }
         else { // wait for all (un-)mount operations to be finished (otherwise, they might be cancelled)
             for(const MountOperation* op : ops) {
                 connect(op, &QObject::destroyed, tabPage, [this, tabPage, viewFrame] {
-                    if(pendingMountOperations().isEmpty()) {
+                    if(ui.sidePane->findChildren<MountOperation*>().isEmpty()) {
                         Settings& settings = static_cast<Application*>(qApp)->settings();
                         if(settings.closeOnUnmount()) {
                             viewFrame->getStackedWidget()->removeWidget(tabPage);
                             tabPage->deleteLater();
                         }
                         else {
-                            tabPage->chdir(Fm::FilePath::homeDir(), viewFrame);
+                            tabPage->chdir(Fm::FilePath::homeDir(), true);
+                            setTabIcon(tabPage);
                             updateUIForCurrentPage();
                         }
                     }
@@ -1265,7 +1390,9 @@ void MainWindow::updateViewMenuForCurrentPage() {
     if(tabPage) {
         // update menus. FIXME: should we move this to another method?
         ui.actionShowHidden->setChecked(tabPage->showHidden());
-        ui.actionPreserveView->setChecked(tabPage->hasCustomizedView());
+        ui.actionPreserveView->setChecked(tabPage->hasCustomizedView() && !tabPage->hasRecursiveCustomizedView());
+        ui.actionPreserveViewRecursive->setChecked(tabPage->hasRecursiveCustomizedView());
+        ui.actionGoToCustomizedViewSource->setVisible(tabPage->hasInheritedCustomizedView());
 
         // view mode
         QAction* modeAction = nullptr;
@@ -1544,7 +1671,8 @@ void MainWindow::onTabPageSortFilterChanged() { // NOTE: This may be called from
     TabPage* tabPage = static_cast<TabPage*>(sender());
     if(tabPage == currentPage()) {
         updateViewMenuForCurrentPage();
-        if(!tabPage->hasCustomizedView()) { // remember sort settings globally
+        ui.sidePane->setShowHidden(tabPage->showHidden());
+        if(!tabPage->hasCustomizedView() && !tabPage->hasInheritedCustomizedView()) { // remember sort settings globally
             Settings& settings = static_cast<Application*>(qApp)->settings();
             settings.setSortColumn(static_cast<Fm::FolderModel::ColumnId>(tabPage->sortColumn()));
             settings.setSortOrder(tabPage->sortOrder());
@@ -1666,7 +1794,7 @@ void MainWindow::on_actionCut_triggered() {
 }
 
 void MainWindow::on_actionPaste_triggered() {
-    pasteFilesFromClipboard(currentPage()->path(), this);
+    pasteFilesFromClipboard(currentPage()->path());
 }
 
 void MainWindow::on_actionDelete_triggered() {
@@ -1796,6 +1924,7 @@ void MainWindow::onBackForwardContextMenu(QPoint pos) {
     if(selectedAction) {
         int index = menu.actions().indexOf(selectedAction);
         page->jumpToHistory(index);
+        setTabIcon(page);
         updateUIForCurrentPage();
     }
 }
@@ -1814,7 +1943,7 @@ void MainWindow::tabContextMenu(const QPoint& pos) {
     TabBar* tabBar = static_cast<TabBar*>(sender());
     if(ViewFrame* viewFrame = qobject_cast<ViewFrame*>(tabBar->parentWidget())) {
         int tabNum = viewFrame->getTabBar()->count();
-        if(tabNum <= 1) {
+        if(tabNum < 1) {
             return;
         }
 
@@ -1824,6 +1953,8 @@ void MainWindow::tabContextMenu(const QPoint& pos) {
         }
 
         QMenu menu;
+
+        // tab closing actions
         if(rightClickIndex_ > 0) {
             menu.addAction(ui.actionCloseLeft);
         }
@@ -1834,6 +1965,24 @@ void MainWindow::tabContextMenu(const QPoint& pos) {
                 menu.addAction(ui.actionCloseOther);
             }
         }
+
+        // per-folder actions for the current tab
+        if(viewFrame->getTabBar()->currentIndex() == rightClickIndex_) {
+            menu.addSeparator();
+            QWidgetAction* labelAction = new QWidgetAction(&menu);
+            QLabel *label = new QLabel(QStringLiteral("<center><b>")
+                                    + tr("Customized View Settings")
+                                    + QStringLiteral("</b></center>"));
+            label->setMargin(5);
+            labelAction->setDefaultWidget(label);
+            menu.addAction(labelAction);
+            menu.addAction(ui.actionPreserveView);
+            menu.addAction(ui.actionPreserveViewRecursive);
+            menu.addSeparator();
+            menu.addAction(ui.actionGoToCustomizedViewSource);
+            menu.addAction(ui.actionCleanPerFolderConfig);
+        }
+
         menu.exec(viewFrame->getTabBar()->mapToGlobal(pos));
     }
 }
@@ -1879,37 +2028,38 @@ void MainWindow::focusPathEntry() {
 void MainWindow::dragEnterEvent(QDragEnterEvent* event) {
     if(event->mimeData()->hasFormat(QStringLiteral("application/pcmanfm-qt-tab"))
        // ensure that the tab drag source is ours (and not a root window, for example)
-       && lastActive_ && lastActive_->isActiveWindow()) {
+       && event->source() != nullptr) {
         event->acceptProposedAction();
     }
 }
 
 void MainWindow::dropEvent(QDropEvent* event) {
     if(event->mimeData()->hasFormat(QStringLiteral("application/pcmanfm-qt-tab"))) {
-        dropTab();
+        dropTab(event->source());
     }
     event->acceptProposedAction();
 }
 
-void MainWindow::dropTab() {
-    if(lastActive_ == nullptr // impossible
-       || lastActive_ == this) { // don't drop on the same window
+void MainWindow::dropTab(QObject* source) {
+    QWidget* w = qobject_cast<QWidget*>(source);
+    MainWindow* dragSource = (w == nullptr ? nullptr : qobject_cast<MainWindow*>(w->window()));
+    if (dragSource == this // drop on itself
+        || dragSource == nullptr) {
         activeViewFrame_->getTabBar()->finishMouseMoveEvent();
         return;
     }
 
-    // close the tab in the first window and add
-    // its page to a new tab in the second window
-    TabPage* dropPage = lastActive_->currentPage();
+    // first close the tab in the drag window;
+    // then add its page to a new tab in the drop window
+    TabPage* dropPage = dragSource->currentPage();
     if(dropPage) {
-        disconnect(dropPage, nullptr, lastActive_, nullptr);
+        disconnect(dropPage, nullptr, dragSource, nullptr);
 
         // release mouse before tab removal because otherwise, the source tabbar
         // might not be updated properly with tab reordering during a fast drag-and-drop
-        lastActive_->activeViewFrame_->getTabBar()->releaseMouse();
+        dragSource->activeViewFrame_->getTabBar()->releaseMouse();
 
-        QWidget* page = lastActive_->activeViewFrame_->getStackedWidget()->currentWidget();
-        lastActive_->activeViewFrame_->getStackedWidget()->removeWidget(page);
+        dragSource->activeViewFrame_->getStackedWidget()->removeWidget(dropPage);
         int index = addTabWithPage(dropPage, activeViewFrame_);
         activeViewFrame_->getTabBar()->setCurrentIndex(index);
     }
@@ -1919,7 +2069,8 @@ void MainWindow::dropTab() {
 }
 
 void MainWindow::detachTab() {
-    if (activeViewFrame_->getStackedWidget()->count() == 1) { // don't detach a single tab
+    if (activeViewFrame_->getStackedWidget()->count() == 1 // don't detach a single tab
+        || static_cast<Application*>(qApp)->settings().splitView()) { // may have changed elsewhere
         activeViewFrame_->getTabBar()->finishMouseMoveEvent();
         return;
     }
@@ -1930,9 +2081,7 @@ void MainWindow::detachTab() {
         disconnect(dropPage, nullptr, this, nullptr);
 
         activeViewFrame_->getTabBar()->releaseMouse(); // as in dropTab()
-
-        QWidget* page = activeViewFrame_->getStackedWidget()->currentWidget();
-        activeViewFrame_->getStackedWidget()->removeWidget(page);
+        activeViewFrame_->getStackedWidget()->removeWidget(dropPage);
         MainWindow* newWin = new MainWindow();
         newWin->addTabWithPage(dropPage, newWin->activeViewFrame_);
         newWin->show();
@@ -1942,6 +2091,39 @@ void MainWindow::detachTab() {
     }
 }
 
+void MainWindow::setTabIcon(TabPage* tabPage) {
+    ViewFrame* viewFrame = viewFrameForTabPage(tabPage);
+    if(viewFrame == nullptr) {
+        return;
+    }
+    bool isCustomized = tabPage->hasCustomizedView() || tabPage->hasInheritedCustomizedView();
+    int index = viewFrame->getStackedWidget()->indexOf(tabPage);
+    auto tabBar = viewFrame->getTabBar();
+
+    if(!isCustomized) {
+        if(!tabBar->tabIcon(index).isNull()) {
+            tabBar->setTabIcon(index, QIcon());
+        }
+        return;
+    }
+
+    // set the tab icon of a customized folder to its view mode
+    switch(tabPage->viewMode()) {
+    case Fm::FolderView::IconMode:
+        tabBar->setTabIcon(index, QIcon::fromTheme(QLatin1String("view-list-icons"), style()->standardIcon(QStyle::SP_FileDialogContentsView)));
+        break;
+    case Fm::FolderView::CompactMode:
+        tabBar->setTabIcon(index, QIcon::fromTheme(QLatin1String("view-list-text"), style()->standardIcon(QStyle::SP_FileDialogListView)));
+        break;
+    case Fm::FolderView::DetailedListMode:
+        tabBar->setTabIcon(index, QIcon::fromTheme(QLatin1String("view-list-details"), style()->standardIcon(QStyle::SP_FileDialogDetailedView)));
+        break;
+    case Fm::FolderView::ThumbnailMode:
+        tabBar->setTabIcon(index, QIcon::fromTheme(QLatin1String("view-preview"), style()->standardIcon(QStyle::SP_FileDialogInfoView)));
+        break;
+    }
+}
+
 void MainWindow::updateFromSettings(Settings& settings) {
     // apply settings
 
@@ -1952,6 +2134,45 @@ void MainWindow::updateFromSettings(Settings& settings) {
     // side pane
     ui.sidePane->setIconSize(QSize(settings.sidePaneIconSize(), settings.sidePaneIconSize()));
 
+    // recent files
+    int recentNumber = settings.getRecentFilesNumber();
+    auto actions = ui.menuRecentFiles->actions();
+    int N = actions.isEmpty() ? 0 : actions.size() - 2; // there is a separator and a clear action
+    if(recentNumber > N) {
+        ui.menuRecentFiles->setEnabled(true);
+        QAction* sep = nullptr;
+        if(actions.size() >= 2) {
+            ui.menuRecentFiles->removeAction(ui.actionClearRecent);
+            sep = ui.menuRecentFiles->actions().last();
+            ui.menuRecentFiles->removeAction(sep);
+        }
+        QAction* recentAction = nullptr;
+        for(int i = N; i < recentNumber; ++i) {
+            recentAction = new QAction(this);
+            recentAction->setVisible(false);
+            connect(recentAction, &QAction::triggered, this, &MainWindow::lanunchRecentFile);
+            ui.menuRecentFiles->addAction(recentAction);
+        }
+        if(sep) {
+            ui.menuRecentFiles->addAction(sep);
+        }
+        else {
+            ui.menuRecentFiles->addSeparator();
+        }
+        ui.menuRecentFiles->addAction(ui.actionClearRecent);
+    }
+    else if(recentNumber < N) {
+        for(int i = 0; i < N - recentNumber; ++i) {
+            auto lastAction = ui.menuRecentFiles->actions().at(ui.menuRecentFiles->actions().size() - 3);
+            ui.menuRecentFiles->removeAction(lastAction);
+            delete lastAction;
+        }
+        if(recentNumber == 0) {
+            ui.menuRecentFiles->clear(); // also deletes the separator
+            ui.menuRecentFiles->setEnabled(false);
+        }
+    }
+
     // tabs
     for(int i = 0; i < ui.viewSplitter->count(); ++i) {
         if(ViewFrame* viewFrame = qobject_cast<ViewFrame*>(ui.viewSplitter->widget(i))) {
@@ -2122,4 +2343,26 @@ void MainWindow::onShortcutJumpToTab() {
     }
 }
 
+void MainWindow::on_actionCleanPerFolderConfig_triggered() {
+    QMessageBox::StandardButton r = QMessageBox::question(this,
+                                    tr("Cleaning Folder Settings"),
+                                    tr("Do you want to remove settings of nonexistent folders?\nThey might be useful if those folders are created again."),
+                                    QMessageBox::Yes | QMessageBox::No,
+                                    QMessageBox::No);
+    if(r == QMessageBox::Yes) {
+        Application* app = static_cast<Application*>(qApp);
+        app->cleanPerFolderConfig();
+    }
+}
+
+void MainWindow::openFolderAndSelectFles(const Fm::FilePathList& files) {
+    if(!files.empty()) {
+        if(auto path = files.front().parent()) {
+            TabPage* newPage = new TabPage(this);
+            addTabWithPage(newPage, activeViewFrame_, std::move(path));
+            newPage->setFilesToSelect(files);
+        }
+    }
+}
+
 }
