@@ -32,7 +32,7 @@
 
 namespace PCManFM {
 
-PreferencesDialog::PreferencesDialog(QString activePage, QWidget* parent):
+PreferencesDialog::PreferencesDialog(const QString& activePage, QWidget* parent):
     QDialog(parent) {
     ui.setupUi(this);
     setAttribute(Qt::WA_DeleteOnClose);
@@ -168,6 +168,7 @@ void PreferencesDialog::initDisplayPage(Settings& settings) {
     ui.showFullNames->setChecked(settings.showFullNames());
     ui.shadowHidden->setChecked(settings.shadowHidden());
     ui.noItemTooltip->setChecked(settings.noItemTooltip());
+    ui.noScrollPerPixel->setChecked(!settings.scrollPerPixel());
 
     // app restart warning
     connect(ui.showFullNames, &QAbstractButton::toggled, [this, &settings] (bool checked) {
@@ -226,6 +227,7 @@ void PreferencesDialog::initBehaviorPage(Settings& settings) {
     ui.quickExec->setChecked(settings.quickExec());
     ui.selectNewFiles->setChecked(settings.selectNewFiles());
     ui.singleWindowMode->setChecked(settings.singleWindowMode());
+    ui.recentFilesSpinBox->setValue(settings.getRecentFilesNumber());
 
     // app restart warning
     connect(ui.quickExec, &QAbstractButton::toggled, [this, &settings] (bool checked) {
@@ -315,6 +317,7 @@ void PreferencesDialog::applyDisplayPage(Settings& settings) {
     settings.setShowFullNames(ui.showFullNames->isChecked());
     settings.setShadowHidden(ui.shadowHidden->isChecked());
     settings.setNoItemTooltip(ui.noItemTooltip->isChecked());
+    settings.setScrollPerPixel(!ui.noScrollPerPixel->isChecked());
     settings.setFolderViewCellMargins(QSize(ui.hMargin->value(), ui.vMargin->value()));
 }
 
@@ -349,6 +352,7 @@ void PreferencesDialog::applyBehaviorPage(Settings& settings) {
     settings.setQuickExec(ui.quickExec->isChecked());
     settings.setSelectNewFiles(ui.selectNewFiles->isChecked());
     settings.setSingleWindowMode(ui.singleWindowMode->isChecked());
+    settings.setRecentFilesNumber(ui.recentFilesSpinBox->value());
 }
 
 void PreferencesDialog::applyThumbnailPage(Settings& settings) {
@@ -398,7 +402,7 @@ void PreferencesDialog::accept() {
     QDialog::accept();
 }
 
-void PreferencesDialog::selectPage(QString name) {
+void PreferencesDialog::selectPage(const QString& name) {
     if(!name.isEmpty()) {
         QWidget* page = findChild<QWidget*>(name + QStringLiteral("Page"));
         if(page) {
