@@ -124,19 +124,25 @@ Settings::Settings():
     showFullNames_(true),
     shadowHidden_(true),
     noItemTooltip_(false),
+    scrollPerPixel_(true),
     bigIconSize_(48),
     smallIconSize_(24),
     sidePaneIconSize_(24),
     thumbnailIconSize_(128),
+    onlyUserTemplates_(false),
+    templateTypeOnce_(false),
+    templateRunApp_(false),
     folderViewCellMargins_(QSize(3, 3)),
     desktopCellMargins_(QSize(3, 1)),
     openWithDefaultFileManager_(false),
+    allSticky_(false),
     searchNameCaseInsensitive_(false),
     searchContentCaseInsensitive_(false),
     searchNameRegexp_(true),
     searchContentRegexp_(true),
     searchRecursive_(false),
-    searchhHidden_(false) {
+    searchhHidden_(false),
+    recentFilesNumber_(0) {
 }
 
 Settings::~Settings() = default;
@@ -182,12 +188,16 @@ QString Settings::profileDir(QString profile, bool useFallback) {
 bool Settings::load(QString profile) {
     profileName_ = profile;
     QString fileName = profileDir(profile, true) + QStringLiteral("/settings.conf");
-    return loadFile(fileName);
+    bool ret = loadFile(fileName);
+    loadRecentFiles();
+    return ret;
 }
 
 bool Settings::save(QString profile) {
     QString fileName = profileDir(profile.isEmpty() ? profileName_ : profile) + QStringLiteral("/settings.conf");
-    return saveFile(fileName);
+    bool ret = saveFile(fileName);
+    saveRecentFiles();
+    return ret;
 }
 
 bool Settings::loadFile(QString filePath) {
@@ -223,6 +233,7 @@ bool Settings::loadFile(QString filePath) {
     confirmTrash_ = settings.value(QStringLiteral("ConfirmTrash"), false).toBool();
     setQuickExec(settings.value(QStringLiteral("QuickExec"), false).toBool());
     selectNewFiles_ = settings.value(QStringLiteral("SelectNewFiles"), false).toBool();
+    recentFilesNumber_ = qBound(0, settings.value(QStringLiteral("RecentFilesNumber"), 0).toInt(), 50);
     settings.endGroup();
 
     settings.beginGroup(QStringLiteral("Desktop"));
@@ -258,6 +269,7 @@ bool Settings::loadFile(QString filePath) {
     desktopCellMargins_ = (settings.value(QStringLiteral("DesktopCellMargins"), QSize(3, 1)).toSize()
                            .expandedTo(QSize(0, 0))).boundedTo(QSize(48, 48));
     openWithDefaultFileManager_ = settings.value(QStringLiteral("OpenWithDefaultFileManager"), false).toBool();
+    allSticky_ = settings.value(QStringLiteral("AllSticky"), false).toBool();
     settings.endGroup();
 
     settings.beginGroup(QStringLiteral("Volume"));
@@ -288,6 +300,7 @@ bool Settings::loadFile(QString filePath) {
     showFullNames_ = settings.value(QStringLiteral("ShowFullNames"), true).toBool();
     shadowHidden_ = settings.value(QStringLiteral("ShadowHidden"), true).toBool();
     noItemTooltip_ = settings.value(QStringLiteral("NoItemTooltip"), false).toBool();
+    scrollPerPixel_ = settings.value(QStringLiteral("ScrollPerPixel"), true).toBool();
 
     // override config in libfm's FmConfig
     bigIconSize_ = toIconSize(settings.value(QStringLiteral("BigIconSize"), 48).toInt(), Big);
@@ -306,11 +319,7 @@ bool Settings::loadFile(QString filePath) {
 
     settings.beginGroup(QStringLiteral("Places"));
     QStringList hiddenPlacesList = settings.value(QStringLiteral("HiddenPlaces")).toStringList();
-#if (QT_VERSION >= QT_VERSION_CHECK(5,14,0))
     hiddenPlaces_ = QSet<QString>(hiddenPlacesList.begin(), hiddenPlacesList.end());
-#else
-    hiddenPlaces_ = hiddenPlacesList.toSet();
-#endif
     settings.endGroup();
 
     settings.beginGroup(QStringLiteral("Window"));
@@ -374,6 +383,7 @@ bool Settings::saveFile(QString filePath) {
     settings.setValue(QStringLiteral("ConfirmTrash"), confirmTrash_);
     settings.setValue(QStringLiteral("QuickExec"), quickExec_);
     settings.setValue(QStringLiteral("SelectNewFiles"), selectNewFiles_);
+    settings.setValue(QStringLiteral("RecentFilesNumber"), recentFilesNumber_);
     settings.endGroup();
 
     settings.beginGroup(QStringLiteral("Desktop"));
@@ -401,6 +411,7 @@ bool Settings::saveFile(QString filePath) {
     settings.setValue(QStringLiteral("SortHiddenLast"), desktopSortHiddenLast_);
     settings.setValue(QStringLiteral("DesktopCellMargins"), desktopCellMargins_);
     settings.setValue(QStringLiteral("OpenWithDefaultFileManager"), openWithDefaultFileManager_);
+    settings.setValue(QStringLiteral("AllSticky"), allSticky_);
     settings.endGroup();
 
     settings.beginGroup(QStringLiteral("Volume"));
@@ -431,6 +442,7 @@ bool Settings::saveFile(QString filePath) {
     settings.setValue(QStringLiteral("ShowFullNames"), showFullNames_);
     settings.setValue(QStringLiteral("ShadowHidden"), shadowHidden_);
     settings.setValue(QStringLiteral("NoItemTooltip"), noItemTooltip_);
+    settings.setValue(QStringLiteral("ScrollPerPixel"), scrollPerPixel_);
 
     // override config in libfm's FmConfig
     settings.setValue(QStringLiteral("BigIconSize"), bigIconSize_);
@@ -448,11 +460,7 @@ bool Settings::saveFile(QString filePath) {
     settings.endGroup();
 
     settings.beginGroup(QStringLiteral("Places"));
-#if (QT_VERSION >= QT_VERSION_CHECK(5,14,0))
     QStringList hiddenPlacesList(hiddenPlaces_.begin(), hiddenPlaces_.end());
-#else
-    QStringList hiddenPlacesList = hiddenPlaces_.values();
-#endif
     settings.setValue(QStringLiteral("HiddenPlaces"), hiddenPlacesList);
     settings.endGroup();
 
@@ -491,6 +499,55 @@ bool Settings::saveFile(QString filePath) {
     return true;
 }
 
+void Settings::setRecentFilesNumber(int n) {
+    recentFilesNumber_ = qBound(0, n, 50);
+    if(recentFilesNumber_ == 0) {
+        clearRecentFiles();
+    }
+}
+
+void Settings::clearRecentFiles() {
+    recentFiles_.clear();
+    saveRecentFiles();
+}
+
+void Settings::addRecentFile(const QString& file) {
+    if(recentFilesNumber_ > 0) {
+        recentFiles_.removeAll(file);
+        recentFiles_.prepend(file);
+        while(recentFiles_.size() > recentFilesNumber_)
+            recentFiles_.removeLast();
+    }
+}
+
+void Settings::loadRecentFiles() {
+    if(recentFilesNumber_ == 0) {
+        return;
+    }
+    // load only from the user-specific ppath
+    QString fileName = profileDir(profileName_) + QStringLiteral("/recent-files.conf");
+    QSettings settings(fileName, QSettings::IniFormat);
+
+    settings.beginGroup(QStringLiteral("Recent"));
+    recentFiles_ = settings.value(QStringLiteral("Files")).toStringList();
+    settings.endGroup();
+
+    recentFiles_.removeAll(QString());
+    recentFiles_.removeDuplicates();
+    while(recentFiles_.count() > recentFilesNumber_) {
+        recentFiles_.removeLast();
+    }
+}
+
+void Settings::saveRecentFiles() {
+    QString fileName = profileDir(profileName_) + QStringLiteral("/recent-files.conf");
+    QSettings settings(fileName, QSettings::IniFormat);
+
+    settings.beginGroup(QStringLiteral("Recent"));
+    settings.setValue(QStringLiteral("Files"), recentFiles_);
+    settings.endGroup();
+}
+
 const QList<int> & Settings::iconSizes(IconType type) {
     static const QList<int> sizes_big = {96, 72, 64, 48, 32};
     static const QList<int> sizes_thumbnail = {256, 224, 192, 160, 128, 96, 64};
@@ -750,8 +807,30 @@ void Settings::setTerminal(QString terminalCommand) {
 FolderSettings Settings::loadFolderSettings(const Fm::FilePath& path) const {
     FolderSettings settings;
     Fm::FolderConfig cfg(path);
-    if(cfg.isEmpty()) {
-        // the folder is not customized; use the general settings
+    bool customized = !cfg.isEmpty();
+    Fm::FilePath inheritedPath;
+    if(!customized
+       && !path.isParentOf(path)) { // WARNING: menu://applications/ is its own parent
+        inheritedPath = path.parent();
+        while(inheritedPath.isValid()) {
+            Fm::GErrorPtr err;
+            cfg.close(err);
+            cfg.open(inheritedPath);
+            if(!cfg.isEmpty()) {
+                bool recursive;
+                if(cfg.getBoolean("Recursive", &recursive) && recursive) {
+                    break;
+                }
+            }
+            if(inheritedPath.isParentOf(inheritedPath)) {
+                inheritedPath = Fm::FilePath(); // invalidate it
+                break;
+            }
+            inheritedPath = inheritedPath.parent();
+        }
+    }
+    if(!customized && !inheritedPath.isValid()) {
+        // the folder is not customized and does not inherit settings; use the general settings
         settings.setSortOrder(sortOrder());
         settings.setSortColumn(sortColumn());
         settings.setViewMode(viewMode());
@@ -761,8 +840,13 @@ FolderSettings Settings::loadFolderSettings(const Fm::FilePath& path) const {
         settings.setSortCaseSensitive(sortCaseSensitive());
     }
     else {
-        // the folder is customized; load folder-specific settings
-        settings.setCustomized(true);
+        // either the folder is customized or it inherits settings; load folder-specific settings
+        if(!inheritedPath.isValid()) {
+            settings.setCustomized(true);
+        }
+        else {
+            settings.seInheritedPath(inheritedPath);
+        }
 
         char* str;
         // load sorting
@@ -804,6 +888,11 @@ FolderSettings Settings::loadFolderSettings(const Fm::FilePath& path) const {
         if(cfg.getBoolean("SortCaseSensitive", &case_sensitive)) {
             settings.setSortCaseSensitive(case_sensitive);
         }
+
+        bool recursive;
+        if(cfg.getBoolean("Recursive", &recursive)) {
+            settings.setRecursive(recursive);
+        }
     }
     return settings;
 }
@@ -822,6 +911,7 @@ void Settings::saveFolderSettings(const Fm::FilePath& path, const FolderSettings
         cfg.setBoolean("SortFolderFirst", settings.sortFolderFirst());
         cfg.setBoolean("SortHiddenLast", settings.sortHiddenLast());
         cfg.setBoolean("SortCaseSensitive", settings.sortCaseSensitive());
+        cfg.setBoolean("Recursive", settings.recursive());
     }
 }
 
