@@ -51,7 +51,8 @@ public:
         showHidden_(false),
         sortFolderFirst_(true),
         sortHiddenLast_(false),
-        sortCaseSensitive_(true) {
+        sortCaseSensitive_(true),
+        recursive_(false) {
     }
 
     bool isCustomized() const {
@@ -118,6 +119,23 @@ public:
         sortCaseSensitive_ = value;
     }
 
+    bool recursive() const {
+        return recursive_;
+    }
+
+    void setRecursive(bool value) {
+        recursive_ = value;
+    }
+
+    Fm::FilePath inheritedPath() const {
+        return inheritedPath_;
+    }
+
+    void seInheritedPath(const Fm::FilePath& path) {
+        inheritedPath_ = std::move(path);
+    }
+
+
 private:
     bool isCustomized_;
     Qt::SortOrder sortOrder_;
@@ -127,6 +145,8 @@ private:
     bool sortFolderFirst_;
     bool sortHiddenLast_;
     bool sortCaseSensitive_;
+    bool recursive_;
+    Fm::FilePath inheritedPath_;
     // columns?
 };
 
@@ -146,9 +166,6 @@ public:
     bool load(QString profile = QStringLiteral("default"));
     bool save(QString profile = QString());
 
-    bool loadFile(QString filePath);
-    bool saveFile(QString filePath);
-
     static QString xdgUserConfigDir();
     static const QList<int> & iconSizes(IconType type);
 
@@ -257,7 +274,7 @@ public:
         return wallpaper_;
     }
 
-    void setWallpaper(QString wallpaper) {
+    void setWallpaper(const QString& wallpaper) {
         wallpaper_ = wallpaper;
     }
 
@@ -779,6 +796,14 @@ public:
         openWithDefaultFileManager_ = open;
     }
 
+    bool allSticky() const {
+        return allSticky_;
+    }
+
+    void setAllSticky(bool sticky) {
+        allSticky_ = sticky;
+    }
+
     bool showThumbnails() {
         return showThumbnails_;
     }
@@ -858,6 +883,14 @@ public:
         noItemTooltip_ = noTooltip;
     }
 
+    bool scrollPerPixel() const {
+        return scrollPerPixel_;
+    }
+
+    void setScrollPerPixel(bool perPixel) {
+        scrollPerPixel_ = perPixel;
+    }
+
     bool onlyUserTemplates() const {
         return onlyUserTemplates_;
     }
@@ -970,9 +1003,27 @@ public:
         }
     }
 
+    int getRecentFilesNumber() const {
+        return recentFilesNumber_;
+    }
+    void setRecentFilesNumber(int n);
+
+    QStringList getRecentFiles() const {
+        return recentFiles_;
+    }
+    void clearRecentFiles();
+    void addRecentFile(const QString& file);
+
+    void loadRecentFiles();
+    void saveRecentFiles();
+
 private:
+    bool loadFile(QString filePath);
+    bool saveFile(QString filePath);
+
     int toIconSize(int size, IconType type) const;
 
+private:
     QString profileName_;
     bool supportTrash_;
 
@@ -1059,6 +1110,7 @@ private:
     bool showFullNames_;
     bool shadowHidden_;
     bool noItemTooltip_;
+    bool scrollPerPixel_;
 
     QSet<QString> hiddenPlaces_;
 
@@ -1076,6 +1128,8 @@ private:
 
     bool openWithDefaultFileManager_;
 
+    bool allSticky_;
+
     // search settings
     bool searchNameCaseInsensitive_;
     bool searchContentCaseInsensitive_;
@@ -1087,6 +1141,10 @@ private:
     // detailed list columns
     QList<QVariant> customColumnWidths_;
     QList<QVariant> hiddenColumns_;
+
+    // recent files
+    int recentFilesNumber_;
+    QStringList recentFiles_;
 };
 
 }
