@@ -41,7 +41,7 @@ View::~View() = default;
 
 void View::onFileClicked(int type, const std::shared_ptr<const Fm::FileInfo>& fileInfo) {
     if(type == MiddleClick) {
-        if(fileInfo->isDir()) {
+        if(fileInfo && fileInfo->isDir()) {
             // fileInfo->path() shouldn't be used directly because
             // it won't work in places like computer:/// or network:///
             Fm::FileInfoList files;
@@ -146,7 +146,17 @@ void View::prepareFileMenu(Fm::FileMenu* menu) {
     }
 }
 
-void View::prepareFolderMenu(Fm::FolderMenu* /*menu*/) {
+void View::prepareFolderMenu(Fm::FolderMenu* menu) {
+    auto folder = folderInfo();
+    if(folder && folder->isNative()) {
+        QAction *action = new QAction(QIcon::fromTheme(QStringLiteral("utilities-terminal")), tr("Open in Termina&l"), menu);
+        connect(action, &QAction::triggered, this, [folder] {
+            Application* app = static_cast<Application*>(qApp);
+            app->openFolderInTerminal(folder->path());
+        });
+        menu->insertAction(menu->createAction(), action);
+        menu->insertSeparator(menu->createAction());
+    }
 }
 
 void View::updateFromSettings(Settings& settings) {
@@ -162,6 +172,8 @@ void View::updateFromSettings(Settings& settings) {
 
     setCtrlRightClick(settings.ctrlRightClick());
 
+    setScrollPerPixel(settings.scrollPerPixel());
+
     Fm::ProxyFolderModel* proxyModel = model();
     if(proxyModel) {
         proxyModel->setShowThumbnails(settings.showThumbnails());
